@@ -5,9 +5,9 @@ package Iterator::Flex::Array;
 use strict;
 use warnings;
 
-our $VERSION = '0.30';
+our $VERSION = '0.31';
 
-use Iterator::Flex::Utils ':IterAttrs';
+use Iterator::Flex::Utils ':IterAttrs', ':IterStates', 'throw_failure';
 use Ref::Util;
 use namespace::clean;
 use experimental 'signatures';
@@ -44,7 +44,7 @@ use parent 'Iterator::Flex::Base';
 
 
 sub new ( $class, $array, $pars = {} ) {
-    $class->_throw( parameter => 'argument must be an ARRAY reference' )
+    throw_failure( parameter => 'argument must be an ARRAY reference' )
       unless Ref::Util::is_arrayref( $array );
 
     $class->SUPER::new( { array => $array }, $pars );
@@ -53,34 +53,39 @@ sub new ( $class, $array, $pars = {} ) {
 
 sub construct ( $class, $state ) {
 
-    $class->_throw( parameter => q{'state' parameter must be a HASH reference} )
+    throw_failure( parameter => q{'state' parameter must be a HASH reference} )
       unless Ref::Util::is_hashref( $state );
 
     my ( $arr, $prev, $current, $next )
       = @{$state}{qw[ array prev current next ]};
 
-    $class->_throw( parameter => q{state 'array' parameter must be a HASH reference} )
+    throw_failure( parameter => q{state 'array' parameter must be a HASH reference} )
       unless Ref::Util::is_arrayref( $arr );
 
     my $len = @$arr;
 
     $next = 0 unless defined $next;
 
-    $class->_throw( parameter => q{illegal value for state 'prev' argument} )
+    throw_failure( parameter => q{illegal value for state 'prev' argument} )
       if defined $prev && ( $prev < 0 || $prev >= $len );
 
-    $class->_throw( parameter => q{illegal value for state 'current' argument} )
+    throw_failure( parameter => q{illegal value for state 'current' argument} )
       if defined $current && ( $current < 0 || $current >= $len );
 
-    $class->_throw( parameter => q{illegal value for state 'next' argument} )
+    throw_failure( parameter => q{illegal value for state 'next' argument} )
       if $next < 0 || $next > $len;
 
     my $self;
+    my $iterator_state;
 
     return {
 
+        ( +_NAME ) => 'iarray',
+
         ( +_SELF ) => \$self,
 
+        ( +STATE ) => \$iterator_state,
+
         ( +RESET ) => sub {
             $prev = $current = undef;
             $next = 0;
@@ -102,7 +107,7 @@ sub construct ( $class, $state ) {
             if ( $next == $len ) {
                 # if first time through, set prev
                 $prev = $current
-                  if !$self->is_exhausted;
+                  if $iterator_state != IterState_EXHAUSTED;
                 return $current = $self->signal_exhaustion;
             }
             $prev    = $current;
@@ -127,7 +132,7 @@ sub construct ( $class, $state ) {
 
 
 __PACKAGE__->_add_roles( qw[
-      State::Registry
+      State::Closure
       Next::ClosedSelf
       Rewind::Closure
       Reset::Closure
@@ -160,7 +165,7 @@ Iterator::Flex::Array - Array Iterator Class
 
 =head1 VERSION
 
-version 0.30
+version 0.31
 
 =head1 METHODS
 
